from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


SETTINGS_TYPES = {
    "RECAPTCHA_DOMAIN": str,
    "RECAPTCHA_PRIVATE_KEY": str,
    "RECAPTCHA_PROXY": dict,
    "RECAPTCHA_PUBLIC_KEY": str,
    "RECAPTCHA_VERIFY_REQUEST_TIMEOUT": int,
}

# Validate settings types.
for variable, instance_type in SETTINGS_TYPES.items():
    if hasattr(settings, variable) \
            and not isinstance(getattr(settings, variable), instance_type):
        raise ImproperlyConfigured(
            "Setting %s is not of type" % variable, instance_type
        )

default_app_config = "captcha.apps.CaptchaConfig"



import logging
import os
import socket
import sys
import warnings

from django import forms
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.exceptions import ValidationError
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from captcha import client
from captcha._compat import HTTPError, urlencode
from captcha.constants import TEST_PRIVATE_KEY, TEST_PUBLIC_KEY
from captcha.widgets import ReCaptchaV2Checkbox, ReCaptchaBase


logger = logging.getLogger(__name__)


class ReCaptchaField(forms.CharField):
    widget = ReCaptchaV2Checkbox
    default_error_messages = {
        "captcha_invalid": _("Error verifying reCAPTCHA, please try again."),
        "captcha_error": _("Error verifying reCAPTCHA, please try again."),
    }

    def __init__(self, public_key=None, private_key=None, *args, **kwargs):
        """
        ReCaptchaField can accepts attributes which is a dictionary of
        attributes to be passed to the ReCaptcha widget class. The widget will
        loop over any options added and create the RecaptchaOptions
        JavaScript variables as specified in
        https://developers.google.com/recaptcha/docs/display#render_param
        """
        super(ReCaptchaField, self).__init__(*args, **kwargs)

        if not isinstance(self.widget, ReCaptchaBase):
            raise ImproperlyConfigured(
                "captcha.fields.ReCaptchaField.widget"
                " must be a subclass of captcha.widgets.ReCaptchaBase"
            )

        # reCAPTCHA fields are always required.
        self.required = True

        # Setup instance variables.
        self.private_key = private_key or getattr(
            settings, "RECAPTCHA_PRIVATE_KEY", TEST_PRIVATE_KEY)
        self.public_key = public_key or getattr(
            settings, "RECAPTCHA_PUBLIC_KEY", TEST_PUBLIC_KEY)

        # Update widget attrs with data-sitekey.
        self.widget.attrs["data-sitekey"] = self.public_key

    def get_remote_ip(self):
        f = sys._getframe()
        while f:
            request = f.f_locals.get("request")
            if request:
                remote_ip = request.META.get("REMOTE_ADDR", "")
                forwarded_ip = request.META.get("HTTP_X_FORWARDED_FOR", "")
                ip = remote_ip if not forwarded_ip else forwarded_ip
                return ip
            f = f.f_back

    def validate(self, value):
        super(ReCaptchaField, self).validate(value)

        try:
            check_captcha = client.submit(
                recaptcha_response=value,
                private_key=self.private_key,
                remoteip=self.get_remote_ip(),
            )

        except HTTPError:  # Catch timeouts, etc
            raise ValidationError(
                self.error_messages["captcha_error"],
                code="captcha_error"
            )

        if not check_captcha.is_valid:
            logger.error(
                "ReCAPTCHA validation failed due to: %s" %
                check_captcha.error_codes
            )
            raise ValidationError(
                self.error_messages["captcha_invalid"],
                code="captcha_invalid"
            )



DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.sqlite',
    }
}

INSTALLED_APPS = [
    'captcha',
]

RECAPTCHA_PRIVATE_KEY = 'privkey'
RECAPTCHA_PUBLIC_KEY = 'pubkey'

SECRET_KEY = 'SECRET_KEY'



import os
import uuid
import warnings

try:
    from unittest.mock import patch, PropertyMock, MagicMock
except ImportError:
    from mock import patch, PropertyMock, MagicMock

from django import forms
from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase, override_settings

from captcha import fields, widgets, constants
from captcha._compat import HTTPError
from captcha.client import RecaptchaResponse


class DefaultForm(forms.Form):
    captcha = fields.ReCaptchaField()


class TestFields(TestCase):

    @patch("captcha.fields.client.submit")
    def test_client_success_response(self, mocked_submit):
        mocked_submit.return_value = RecaptchaResponse(is_valid=True)
        form_params = {"g-recaptcha-response": "PASSED"}
        form = DefaultForm(form_params)
        self.assertTrue(form.is_valid())

    @patch("captcha.fields.client.submit")
    def test_client_failure_response(self, mocked_submit):
        mocked_submit.return_value = RecaptchaResponse(
            is_valid=False, error_codes=["410"]
        )
        form_params = {"g-recaptcha-response": "PASSED"}
        form = DefaultForm(form_params)
        self.assertFalse(form.is_valid())

    def test_widget_check(self):
        with self.assertRaises(ImproperlyConfigured):
            class ImporperForm(forms.Form):
                captcha = fields.ReCaptchaField(widget=forms.Textarea)

    @patch("captcha.fields.client.submit")
    def test_field_instantiate_values(self, mocked_submit):
        mocked_submit.return_value = RecaptchaResponse(is_valid=True)

        class NonDefaultForm(forms.Form):
            captcha = fields.ReCaptchaField(
                private_key="NewUpdatedKey", public_key="NewPubKey"
            )

        form_params = {"g-recaptcha-response": "PASSED"}
        form = NonDefaultForm(form_params)
        self.assertTrue(form.is_valid())
        mocked_submit.assert_called_with(
            private_key="NewUpdatedKey",
            recaptcha_response="PASSED",
            remoteip=None,
        )
        html = form.as_p()
        self.assertIn('data-sitekey="NewPubKey"', html)

    @patch("captcha.client.recaptcha_request")
    def test_field_captcha_errors(self, mocked_response):
        read_mock = MagicMock()
        read_mock.read.return_value = b'{"success": false, "error-codes":' \
            b'["invalid-input-response", "invalid-input-secret"]}'
        mocked_response.return_value = read_mock
        form_params = {"g-recaptcha-response": "PASSED"}
        form = DefaultForm(form_params)
        self.assertFalse(form.is_valid())
        self.assertEqual(
            form.errors["captcha"],
            ["Error verifying reCAPTCHA, please try again."]
        )

        mocked_response.side_effect = HTTPError(
            url="https://www.google.com/recaptcha/api/siteverify",
            code=410,
            fp=None,
            msg="Oops",
            hdrs=""
        )
        form = DefaultForm(form_params)
        self.assertFalse(form.is_valid())
        self.assertEqual(
            form.errors["captcha"],
            ["Error verifying reCAPTCHA, please try again."]
        )


class TestWidgets(TestCase):
    @patch("captcha.widgets.uuid.UUID.hex", new_callable=PropertyMock)
    def test_default_v2_checkbox_html(self, mocked_uuid):
        test_hex = "928e8e017b114e1b9d3a3e877cfc5844"
        mocked_uuid.return_value = test_hex

        class DefaultCheckForm(forms.Form):
            captcha = fields.ReCaptchaField()

        form = DefaultCheckForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.google.com/recaptcha/api.js'
            '"></script>',
            html
        )
        self.assertIn('data-size="normal"', html)
        self.assertIn('class="g-recaptcha"', html)
        self.assertIn('data-callback="onSubmit_%s"' % test_hex, html)
        self.assertIn("required", html)
        self.assertIn('data-widget-uuid="%s"' % test_hex, html)
        self.assertIn('data-sitekey="pubkey"', html)
        self.assertIn("var onSubmit_%s = function(token) {" % test_hex, html)

    @patch("captcha.widgets.uuid.UUID.hex", new_callable=PropertyMock)
    def test_v2_checkbox_attribute_changes_html(self, mocked_uuid):
        test_hex = "e83ccae286ad4784bd47f7ddc40cfd6f"
        mocked_uuid.return_value = test_hex

        class CheckboxAttrForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV2Checkbox(
                    attrs={
                        "data-theme": "dark",
                        "data-callback": "customCallback",
                        "data-size": "compact"
                    },
                    api_params={
                        "hl": "af"
                    }
                )
            )

        form = CheckboxAttrForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.google.com/recaptcha/api.js'
            '?hl=af"></script>',
            html
        )
        self.assertIn('data-theme="dark"', html)
        self.assertNotIn('data-callback="onSubmit_%s"' % test_hex, html)
        self.assertIn('data-callback="customCallback"', html)
        self.assertIn('data-size="compact"', html)
        self.assertIn('class="g-recaptcha"', html)
        self.assertIn("required", html)
        self.assertIn('data-widget-uuid="%s"' % test_hex, html)
        self.assertIn('data-sitekey="pubkey"', html)
        self.assertIn("var onSubmit_%s = function(token) {" % test_hex, html)

    @override_settings(RECAPTCHA_DOMAIN="www.recaptcha.net")
    def test_default_v2_checkbox_domain_html(self):
        class DomainForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV2Checkbox()
            )

        form = DomainForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.recaptcha.net/recaptcha/api.js">'
            '</script>',
            html
        )

    @patch("captcha.widgets.uuid.UUID.hex", new_callable=PropertyMock)
    def test_default_v2_invisible_html(self, mocked_uuid):
        test_hex = "72f853eb8b7e4022b808be0f5c3bc297"
        mocked_uuid.return_value = test_hex

        class InvisForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV2Invisible()
            )

        form = InvisForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.google.com/recaptcha/api.js'
            '"></script>',
            html
        )
        self.assertIn('data-size="invisible"', html)
        self.assertIn('data-callback="onSubmit_%s"' % test_hex, html)
        self.assertIn('class="g-recaptcha"', html)
        self.assertIn("required", html)
        self.assertIn('data-widget-uuid="%s"' % test_hex, html)
        self.assertIn('data-sitekey="pubkey"', html)
        self.assertIn("var onSubmit_%s = function(token) {" % test_hex, html)
        self.assertIn("var verifyCaptcha_%s = function(e) {" % test_hex, html)
        self.assertIn('.g-recaptcha[data-widget-uuid="%s"]' % test_hex, html)

    @patch("captcha.widgets.uuid.UUID.hex", new_callable=PropertyMock)
    def test_v2_invisible_attribute_changes_html(self, mocked_uuid):
        test_hex = "8b220c54ddb849b8bb59bda5da57baea"
        mocked_uuid.return_value = test_hex

        class InvisAttrForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV2Invisible(
                    attrs={
                        "data-theme": "dark",
                        "data-callback": "customCallbackInvis",
                        "data-size": "compact"
                    },
                    api_params={
                        "hl": "cl"
                    }
                )
            )

        form = InvisAttrForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.google.com/recaptcha/api.js'
            '?hl=cl"></script>',
            html
        )
        self.assertNotIn('data-size="compact"', html)
        self.assertIn('data-size="invisible"', html)
        self.assertNotIn('data-callback="onSubmit_%s"' % test_hex, html)
        self.assertIn('data-callback="customCallbackInvis"', html)
        self.assertIn('class="g-recaptcha"', html)
        self.assertIn("required", html)
        self.assertIn('data-widget-uuid="%s"' % test_hex, html)
        self.assertIn('data-sitekey="pubkey"', html)
        self.assertIn("var onSubmit_%s = function(token) {" % test_hex, html)
        self.assertIn("var verifyCaptcha_%s = function(e) {" % test_hex, html)
        self.assertIn('.g-recaptcha[data-widget-uuid="%s"]' % test_hex, html)

    @override_settings(RECAPTCHA_DOMAIN="www.recaptcha.net")
    def test_default_v2_invisible_domain_html(self):
        class InvisDomainForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV2Invisible()
            )

        form = InvisDomainForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.recaptcha.net/recaptcha/api.js">'
            '</script>',
            html
        )

    @patch("captcha.widgets.uuid.UUID.hex", new_callable=PropertyMock)
    def test_default_v3_html(self, mocked_uuid):
        test_hex = "c7a86421ca394661acccea374931d260"
        mocked_uuid.return_value = test_hex

        class InvisForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV3()
            )

        form = InvisForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.google.com/recaptcha/api.js'
            '?render=pubkey"></script>',
            html
        )
        self.assertIn('data-size="normal"', html)
        self.assertIn('data-callback="onSubmit_%s"' % test_hex, html)
        self.assertIn('class="g-recaptcha"', html)
        self.assertIn("required", html)
        self.assertIn('data-widget-uuid="%s"' % test_hex, html)
        self.assertIn('data-sitekey="pubkey"', html)
        self.assertIn('.g-recaptcha[data-widget-uuid="%s"]' % test_hex, html)

    @patch("captcha.widgets.uuid.UUID.hex", new_callable=PropertyMock)
    def test_v3_attribute_changes_html(self, mocked_uuid):
        test_hex = "f367f89a797a4985acd986275b3df22f"
        mocked_uuid.return_value = test_hex

        class InvisAttrForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV3(
                    attrs={
                        "data-theme": "dark",
                        "data-callback": "customCallbackInvis",
                        "data-size": "compact"
                    },
                    api_params={
                        "hl": "cl"
                    }
                )
            )

        form = InvisAttrForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.google.com/recaptcha/api.js'
            '?render=pubkey&hl=cl"></script>',
            html
        )
        self.assertIn('data-size="compact"', html)
        self.assertNotIn('data-callback="onSubmit_%s"' % test_hex, html)
        self.assertIn('data-callback="customCallbackInvis"', html)
        self.assertIn('class="g-recaptcha"', html)
        self.assertIn("required", html)
        self.assertIn('data-widget-uuid="%s"' % test_hex, html)
        self.assertIn('data-sitekey="pubkey"', html)
        self.assertIn('.g-recaptcha[data-widget-uuid="%s"]' % test_hex, html)

    @override_settings(RECAPTCHA_DOMAIN="www.recaptcha.net")
    def test_default_v3_domain_html(self):
        class VThreeDomainForm(forms.Form):
            captcha = fields.ReCaptchaField(
                widget=widgets.ReCaptchaV3()
            )

        form = VThreeDomainForm()
        html = form.as_p()
        self.assertIn(
            '<script src="https://www.recaptcha.net/recaptcha/api.js'
            '?render=pubkey"></script>',
            html
        )



from django.conf import settings
from django.core import checks

from captcha.constants import TEST_PRIVATE_KEY, TEST_PUBLIC_KEY


def recaptcha_key_check(app_configs, **kwargs):
    errors = []
    private_key = getattr(
        settings, "RECAPTCHA_PRIVATE_KEY", TEST_PRIVATE_KEY)
    public_key = getattr(
        settings, "RECAPTCHA_PUBLIC_KEY", TEST_PUBLIC_KEY)

    if private_key == TEST_PRIVATE_KEY or \
            public_key == TEST_PUBLIC_KEY:
        errors.extend([checks.Error(
            "RECAPTCHA_PRIVATE_KEY or RECAPTCHA_PUBLIC_KEY is making use"
            " of the Google test keys and will not behave as expected in a"
            " production environment",
            hint="Update settings.RECAPTCHA_PRIVATE_KEY"
            " and/or settings.RECAPTCHA_PUBLIC_KEY. Alternatively this check"
            " can be ignored by adding"
            " `SILENCED_SYSTEM_CHECKS = ['captcha.recaptcha_test_key_error']`"
            " to your settings file.",
            id="captcha.recaptcha_test_key_error"
        )])
    return errors



import sys

PY2 = sys.version_info[0] == 2
if PY2:
    text_type = unicode
    from urllib2 import build_opener, ProxyHandler, Request, urlopen, HTTPError
    from urllib import urlencode
else:
    from urllib.request import build_opener, ProxyHandler, Request, urlopen
    from urllib.parse import urlencode
    from urllib.error import HTTPError
    text_type = str



import warnings


class generic_deprecation(object):
    def __init__(
            self, message, warning_class=DeprecationWarning, stack_level=2):
        self.message = message
        self.warning_class = warning_class
        self.stack_level = stack_level

    def __call__(self, method):
        def wrapped(*args, **kwargs):
            warnings.warn(self.message, self.warning_class, self.stack_level)
            return method(*args, **kwargs)
        return wrapped



import uuid

try:
    from unittest.mock import patch, PropertyMock, MagicMock
except ImportError:
    from mock import patch, PropertyMock, MagicMock

from django import forms
from django.conf import settings
from django.test import TestCase, override_settings

from captcha import client, constants, fields


class DefaultForm(forms.Form):
    captcha = fields.ReCaptchaField()


class TestClient(TestCase):

    @patch("captcha.client.recaptcha_request")
    def test_client_success(self, mocked_response):
        read_mock = MagicMock()
        read_mock.read.return_value = b'{"success": true, "challenge_ts":' \
            b'"2019-01-11T13:57:23Z", "hostname": "testkey.google.com"}'
        mocked_response.return_value = read_mock
        uuid_hex = uuid.uuid4().hex
        response = client.submit(
            uuid_hex,
            "somekey",
            "0.0.0.0",
        )

        # Quick way to test method call without needing to worry about Python 2
        # dicts not being ordered by default.
        self.assertIn("secret=somekey", mocked_response.call_args.__str__())
        self.assertIn(
            "response=%s" % uuid_hex, mocked_response.call_args.__str__()
        )
        self.assertIn("remoteip=0.0.0.0", mocked_response.call_args.__str__())
        self.assertTrue(response.is_valid)

    @patch("captcha.client.recaptcha_request")
    def test_client_failure(self, mocked_response):
        read_mock = MagicMock()
        read_mock.read.return_value = b'{"success": false, "error-codes":' \
            b'["invalid-input-response", "invalid-input-secret"]}'
        mocked_response.return_value = read_mock
        uuid_hex = uuid.uuid4().hex
        response = client.submit(
            uuid_hex,
            "somekey",
            "0.0.0.0",
        )

        # Quick way to test method call without needing to worry about Python 2
        # dicts not being ordered by default.
        self.assertIn("secret=somekey", mocked_response.call_args.__str__())
        self.assertIn(
            "response=%s" % uuid_hex, mocked_response.call_args.__str__()
        )
        self.assertIn("remoteip=0.0.0.0", mocked_response.call_args.__str__())
        self.assertFalse(response.is_valid)
        self.assertEqual(
            response.error_codes.sort(),
            ["invalid-input-response", "invalid-input-secret"].sort()
        )

    @patch("captcha.client.Request")
    @patch("captcha.client.build_opener")
    def test_client_request(self, mocked_builder, mocked_request):
        mock_read = MagicMock()
        mock_read.read.return_value = b'{"success": false, "error-codes":' \
            b'["invalid-input-response", "invalid-input-secret"]}'
        mock_opener = MagicMock()
        mock_opener.open.return_value = mock_read
        mocked_builder.return_value = mock_opener
        form_params = {"g-recaptcha-response": "PASSED"}
        form = DefaultForm(form_params)
        form.is_valid()

        # Quick way to test method call without needing to worry about Python 2
        # dicts not being ordered by default.
        self.assertIn(
            "data=",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "remoteip=None",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "response=PASSED",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "secret=privkey",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "headers=",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "'Content-type': 'application/x-www-form-urlencoded'",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "'User-agent': 'reCAPTCHA Django'",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "url='https://www.google.com/recaptcha/api/siteverify'",
            mocked_request.call_args.__str__()
        )
        mock_opener.open.assert_called_with(mocked_request(), timeout=10)
        mocked_builder.assert_called_with()

    @patch("captcha.client.ProxyHandler")
    @patch("captcha.client.Request")
    @patch("captcha.client.build_opener")
    @override_settings(RECAPTCHA_PROXY={"http": "aaaa.com"})
    def test_client_request_with_proxy_builder(
            self, mocked_builder, mocked_request, mocked_handler):
        mock_read = MagicMock()
        mock_read.read.return_value = b'{"success": false, "error-codes":' \
            b'["invalid-input-response", "invalid-input-secret"]}'
        mock_opener = MagicMock()
        mock_opener.open.return_value = mock_read
        mocked_builder.return_value = mock_opener
        form_params = {"g-recaptcha-response": "PASSED"}
        form = DefaultForm(form_params)
        form.is_valid()

        # Quick way to test method call without needing to worry about Python 2
        # dicts not being ordered by default.
        self.assertIn(
            "data=",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "remoteip=None",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "response=PASSED",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "secret=privkey",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "headers=",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "'Content-type': 'application/x-www-form-urlencoded'",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "'User-agent': 'reCAPTCHA Django'",
            mocked_request.call_args.__str__()
        )
        self.assertIn(
            "url='https://www.google.com/recaptcha/api/siteverify'",
            mocked_request.call_args.__str__()
        )
        mock_opener.open.assert_called_with(mocked_request(), timeout=10)
        mocked_handler.assert_called_with({"http": "aaaa.com"})
        mocked_builder.assert_called_with(mocked_handler())



import json
import uuid

from django.conf import settings
from django.forms import widgets
from django.utils.safestring import mark_safe
from django.utils.translation import get_language

from captcha._compat import urlencode
from captcha.constants import DEFAULT_RECAPTCHA_DOMAIN


class ReCaptchaBase(widgets.Widget):
    """
    Base widget to be used for Google ReCAPTCHA.

    public_key -- String value: can optionally be passed to not make use of the
        project wide Google Site Key.
    """
    recaptcha_response_name = "g-recaptcha-response"

    def __init__(self, api_params=None, *args, **kwargs):
        super(ReCaptchaBase, self).__init__(*args, **kwargs)
        self.uuid = uuid.uuid4().hex
        self.api_params = api_params or {}

    def value_from_datadict(self, data, files, name):
        return data.get(self.recaptcha_response_name, None)

    def get_context(self, name, value, attrs):
        context = super(ReCaptchaBase, self).get_context(name, value, attrs)
        params = urlencode(self.api_params)
        context.update({
            "public_key": self.attrs["data-sitekey"],
            "widget_uuid": self.uuid,
            "api_params": params,
            "recaptcha_domain": getattr(
                settings, "RECAPTCHA_DOMAIN", DEFAULT_RECAPTCHA_DOMAIN
            ),
        })
        return context

    def build_attrs(self, base_attrs, extra_attrs=None):
        attrs = super(ReCaptchaBase, self).build_attrs(base_attrs, extra_attrs)
        attrs["data-widget-uuid"] = self.uuid

        # Support the ability to override some of the Google data attrs.
        attrs["data-callback"] = base_attrs.get(
            "data-callback", "onSubmit_%s" % self.uuid
        )
        attrs["data-size"] = base_attrs.get("data-size", "normal")
        return attrs


class ReCaptchaV2Checkbox(ReCaptchaBase):
    template_name = "captcha/widget_v2_checkbox.html"


class ReCaptchaV2Invisible(ReCaptchaBase):
    template_name = "captcha/widget_v2_invisible.html"

    def build_attrs(self, base_attrs, extra_attrs=None):
        attrs = super(ReCaptchaV2Invisible, self).build_attrs(
            base_attrs, extra_attrs
        )

        # Invisible reCAPTCHA should not have another size
        attrs["data-size"] = "invisible"
        return attrs


class ReCaptchaV3(ReCaptchaBase):
    template_name = "captcha/widget_v3.html"

    def build_attrs(self, base_attrs, extra_attrs=None):
        attrs = super(ReCaptchaV3, self).build_attrs(
            base_attrs, extra_attrs
        )
        return attrs

    def value_from_datadict(self, data, files, name):
        return data.get(name)



from setuptools import setup, find_packages


long_desc = open('README.rst', 'rb').read().decode('utf-8') + '\n\n' + \
            open('AUTHORS.rst', 'rb').read().decode('utf-8') + '\n\n' + \
            open('CHANGELOG.rst', 'rb').read().decode('utf-8')

setup(
    name='django-recaptcha',
    version='2.0.2',
    description='Django recaptcha form field/widget app.',
    long_description=long_desc,
    author='Praekelt Consulting',
    author_email='dev@praekelt.com',
    license='BSD',
    url='http://github.com/praekelt/django-recaptcha',
    packages=find_packages(),
    install_requires=[
        'django>1.11,<2.3',
    ],
    tests_require=[
        'tox',
    ],
    keywords=['django', 'reCAPTCHA', 'reCAPTCHA v2', 'reCAPTCHA v3'],
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 1.11',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    zip_safe=False,
)
